﻿/// @copyright  	Copyright (c) 2014 Toradex AG.
///                 [Software License Agreement]
/// $Author:		khv $
/// $Date: 			$ 2014-06-24 $
/// @brief:			GPSID demo in VC#.

using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Security.Cryptography;
using System.Runtime.InteropServices;

namespace vcsharp_gpsid_demo
{
    /// UTC date and time structure.
    public struct SYSTEMTIME
    {
        public UInt16 year;
        public UInt16 month;
        public UInt16 dayOfWeek;
        public UInt16 day;
        public UInt16 hour;
        public UInt16 minute;
        public UInt16 second;
        public UInt16 milliSecond;
    }

    /// GPS position structure.
    [StructLayout(LayoutKind.Sequential)]
    public struct GPS_POSITION
    {
        public uint version;
        public uint size;
        public uint validFields;
        public uint flags;
        public SYSTEMTIME utcTime;
        public Double latitude;
        public Double longitude;
        public float speed;
        public float heading;
        public Double magnetVariation;
        public float altitudeSeaLevel;
        public float altitudeEllipsoid;
        public int fixQuality;
        public int fixType;
        public int fixSelection;
        public float pdop;
        public float hdop;
        public float vdop;
        public uint satellitesCount;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 12)]
        public uint[] satellitesUsedPrns;
        public uint satellitesInView;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 12)]
        public uint[] satellitesInViewPrns;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 12)]
        public uint[] satellitesInViewElevation;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 12)]
        public uint[] satellitesInViewAzimuth;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 12)]
        public uint[] satellitesInViewSNRatio;
    }

    public partial class Form1 : Form
    {
        [DllImport("gpsapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern IntPtr GPSOpenDevice(
           IntPtr hNewLocationData,
           IntPtr hDeviceStateChange, 
           string szDeviceName, 
           int flags);

        [DllImport("gpsapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int GPSGetPosition(
           IntPtr openHandle,
           ref GPS_POSITION gpsPosition,
           int dwMaximumAge,
           int flags);

        [DllImport("gpsapi.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern int GPSCloseDevice(IntPtr openHandle);

        public const int ERROR_SUCCESS          = 0;
        public const int GPS_VERSION_1          = 1;
        public const int GPS_MAX_SATELLITES     = 12;

        IntPtr retOpenHandle = IntPtr.Zero;
        bool handleClosed = true;
        int retVal = 0;
        IntPtr gpsPosition = IntPtr.Zero;
        GPS_POSITION gpsPositionParameter;
 
        public Form1()
        {
            InitializeComponent();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            gpsPositionParameter.version = GPS_VERSION_1;                               ///< GPS version must be passed to GPSGetPosition function.
            gpsPositionParameter.size = (uint)Marshal.SizeOf(typeof(GPS_POSITION));     ///< Size of GPS_POSITION structure must be passed to GPSGetPosition function.
            retVal = GPSGetPosition(retOpenHandle, ref gpsPositionParameter, 5000, 0);  ///< Gets time and position parameters.
            if (retVal != ERROR_SUCCESS)
            {
                MessageBox.Show("Position detection failed, Error code (winerror.h) = " + retVal, "Error (Stop and Try again)");
                timer1.Enabled = false;
            }
            else
            {
                date.Text = gpsPositionParameter.utcTime.day.ToString("D" + 2);                     ///< UTC date.
                month.Text = gpsPositionParameter.utcTime.month.ToString("D" + 2);                  ///< UTC month.
                year.Text = gpsPositionParameter.utcTime.year.ToString("D" + 4);                    ///< UTC year.
                hour.Text = gpsPositionParameter.utcTime.hour.ToString("D" + 2);                    ///< UTC hour.
                minute.Text = gpsPositionParameter.utcTime.minute.ToString("D" + 2);                ///< UTC minutes.
                second.Text = gpsPositionParameter.utcTime.second.ToString("D" + 2);                ///< UTC seconds.
                secondFraction.Text = gpsPositionParameter.utcTime.milliSecond.ToString("D" + 3);   ///< UTC milliseconds.
                satellitesInView.Text = gpsPositionParameter.satellitesInView.ToString("D" + 2);    ///< Total satellites in range.
                satellitesUsed.Text = gpsPositionParameter.satellitesCount.ToString("D" + 2);       ///< Total satellites used to get the position.
                latitude.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.latitude);    ///< Latitude.
                /// Latitude direction.                                                                                    
                if (gpsPositionParameter.latitude > 0.0)
                {
                    latitudeDirection.Text = "N";
                }
                else if (gpsPositionParameter.latitude < 0.0)
                {
                    latitudeDirection.Text = "S";
                }
                else
                {
                    latitudeDirection.Text = "";
                }
                longitude.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.longitude);    ///< Longitude.
                /// Longitude direction.
                if (gpsPositionParameter.longitude > 0.0)
                {
                    longitudeDirection.Text = "E";
                }
                else if (gpsPositionParameter.longitude < 0.0)
                {
                    longitudeDirection.Text = "W";
                }
                else
                {
                    longitudeDirection.Text = "";
                }
                /// GPS fix type.
                if (gpsPositionParameter.fixType == 0)
                {
                    fixType.Text = "";
                }
                else if (gpsPositionParameter.fixType == 1)
                {
                    fixType.Text = "2D";
                }
                else
                {
                    fixType.Text = "3D";
                }
                /// GPS fix quality.
                if (gpsPositionParameter.fixQuality == 0)
                {
                    fixQuality.Text = "";
                }
                else if (gpsPositionParameter.fixQuality == 1)
                {
                    fixQuality.Text = " GPS";
                }
                else
                {
                    fixQuality.Text = "DGPS";
                }
                /// GPS fix selection.
                if (gpsPositionParameter.fixSelection == 0)
                {
                    fixSelection.Text = "";
                }
                else if (gpsPositionParameter.fixSelection == 1)
                {
                    fixSelection.Text = "  Auto";
                }
                else
                {
                    fixSelection.Text = "Manual";
                }
                altitudeSea.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.altitudeSeaLevel);         ///< Altitude w.r.t. sea level.
                altitudeEllipsoid.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.altitudeEllipsoid);  ///< Altitude w.r.t Ellipsoid.
                heading.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.heading);                      ///< True course over ground.
                magVariation.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.magnetVariation);         ///< Magnetic variation.
                /// Magnetic variation direction.
                if (gpsPositionParameter.magnetVariation > 0.0)
                {
                    magVariationDirection.Text = "E";
                }
                else if (gpsPositionParameter.magnetVariation < 0.0)
                {
                    magVariationDirection.Text = "W";
                }
                else
                {
                    magVariationDirection.Text = "";
                }
                pdop.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.pdop);                ///< Position dilution of precision.
                hdop.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.hdop);                ///< Horizontal dilution of precision.
                vdop.Text = String.Format("{0:0,0.0000000}", gpsPositionParameter.vdop);                ///< Vertical dilution of precision.
                speed.Text = String.Format("{0:0,0.0000000}", (gpsPositionParameter.speed * 1.852));    ///< Speed in KMPH (1 Knot = 1.852 KMPH).
            }
        }

        private void run_Click_1(object sender, EventArgs e)
        {
            if (handleClosed == true)
            {
                try
                {
                    timer1.Interval = Convert.ToInt32(timeMilliSeconds.Text);
                }
                catch
                {
                    MessageBox.Show("Enter Refresh time in milliseconds.", "Error");
                    return;
                }
                if (timer1.Interval < (Int32)300)
                {
                    MessageBox.Show("Minimum recommended time = 300 ms.", "Refresh Time");
                    return;
                }
                retOpenHandle = GPSOpenDevice(IntPtr.Zero, IntPtr.Zero, null, 0);       ///< Opens handle to GPS driver.
                if (retOpenHandle == IntPtr.Zero)
                {
                    MessageBox.Show("Driver access Denied, stop and try again.", "Error");
                    return;
                }
                else
                {
                    handleClosed = false;
                    timer1.Enabled = true;
                }
            }
            else
            {
                MessageBox.Show("Stop and try again.", "Error");
                return;
            }
        }

        private void stop_Click_1(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            if (handleClosed == false)
            {
                GPSCloseDevice(retOpenHandle);          ///< Closes handle to GPS driver.
            }
            handleClosed = true;
        }

        private void Form1_closing(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            if (handleClosed == false)
            {
                GPSCloseDevice(retOpenHandle);          ///< Closes handle to GPS driver.
            }
            handleClosed = true;
        }
    }
}